#!/usr/bin/perl -w

######################################################################################
# mminfo2html.pl
#
# A simple utility designed to convert the XML output of mminfo into a HTML file.
#
# Designed by Preston de Guise for the NetWorker Information Blog.
# You may edit or adjust this script however you wish, but you this preliminary
# comment blurb must not be removed. You may not create a commercial derivative of
# this script.
######################################################################################

######################################################################################
# Modules
######################################################################################
use strict;
use Getopt::Std;
use File::Basename;

######################################################################################
# Global variables
######################################################################################
my $self = basename($0);
my $cssFile = "";
my %entries = ();
my @order = ();
my $title = "mminfo2html Output";
my $version = "1.0";


######################################################################################
# Subroutines
######################################################################################

# usage([@_]) prints standard usage information, any other optional text provided by
# the calling function, and exits the program.
sub usage {
	print <<EOF;
Usage: $self [-h] [-v] [-c css] [-t title]

Where:

	-h		Prints this help and exits.
	-v		Prints the version and exits.
	-c css		Use CSS file 'css' as CSS for formatting.
	-t title	Replace standard title with 'title'.
	
Converts mminfo XML format output into a HTML table.

EOF

	if (@_+0 != 0) {
		my @lines = @_;
		foreach my $line (@lines) {
			my $tmp = $line;
			chomp $tmp;
			print "$tmp\n";
		}
	}
	die "\n";
}

# process_command_line() checks for the various command line options.
sub process_command_line {
	my %opts;
	if (getopts('hvc:t:',\%opts)) {
		usage() if (defined($opts{h}));
		die "$self is v$version\n" if defined($opts{v});
		
		if (defined($opts{c})) {
			if (-f $opts{c}) {
				$cssFile = $opts{c};
			} else {
				usage("CSS file specified (-c file) does not exist or cannot be opened.");
			}
		}
		
		if (defined($opts{t})) {
			$title = $opts{t};
		}
	}
}

# read_xml_input() reads the XML input stream from standard input.
sub read_xml_input {
	my $count = 0;
	my $reportStarted = 0;
	my $haveOrder = 0;
	while (<>) {
		my $line = $_;
		chomp $line;

		if ($line =~ /mminfo-tabular-report/) {
			$reportStarted = 1;
			next;
		}

		next if (!$reportStarted);

		if ($line =~ /^<result>/) {
			# Start of a result entry.
			my $nextLine = "";
			while ($nextLine ne "</result>") {
				my $nextLine = <>;
				chomp $nextLine;
				if ($nextLine =~ "</result>") {
					$haveOrder = 1;
					$count++;
					last;
				}
				if ($nextLine =~ /^<(.*)>(.*)<\/.*>$/) {
					my $field = $1;
					my $data = $2;
					$entries{$count}{$field} = $data;
					if (!$haveOrder) {
						push(@order,$field);
					}
				}
			}
		}
	}
}

# get_css() either populates our CSS reference with the local, standard CSS, or retrieves
# CSS from a file. NB: No attempt is made to validate the CSS provided.
sub get_css {
	my $defaultCSS = "<STYLE type=\"text/css\">
	table {
			border-spacing: 0px;
			border-collapse: collapse;
		}
    th 
        {
            border-top: 1px #000000 solid;      border-bottom: 1px #000000 solid;
            border-left: 1px #808080 solid;     border-right: 1px #808080 solid;
            color: black;                       background: #F0F0F0;    
            font-weight: bold;
            padding: 5px;
        }

    td 
        {
            border-top: 0px #000000 solid;  border-bottom: 1px #000000 solid;
            border-left: 1px #808080 solid;     border-right: 1px #808080 solid; 
            padding: 5px;
        }
       
    body 
        { 
            color: black;   			background: white;
            font-size: 12pt;            font-family: Helvetica; 
        }
        
</STYLE>";

	if ($cssFile ne "") {
		# read and populate a text string with the CSS.
		# return the CSS.
		if (open(CSS,$cssFile)) {
			local $/;
			my $newCSS = <CSS>;
			close(CSS);
			return $newCSS;
		}
	} else {
		return $defaultCSS;
	}
}

# write_html_output() writes the HTML table out to standard output.
sub write_html_output {
	my $css = get_css();
	print <<EOF;
<HTML>
	<HEAD>
	<TITLE>mminfo2html Output</TITLE>
	$css
	</HEAD>

<BODY>
<H1>$title</H1>

<TABLE>
	<TR>
EOF

	foreach my $field (@order) {
		print "\t\t<TH><B>" . ucfirst($field) . "</B></TH>\n"
	}
	print "\t</TR>\n";

	foreach my $entry (sort {$a <=> $b} keys %entries) {
		print "\t<TR>\n";
		foreach my $field (@order) {
			print "\t<TD>" . $entries{$entry}{$field} . "</TD>\n";
		}
		print "\t</TR>\n";
	}
	print <<EOF;
</TABLE>
</BODY>
</HTML>
EOF
}

######################################################################################
# Main
######################################################################################

process_command_line();
read_xml_input();
write_html_output();

